@@ -55,7 +55,7 @@ _gdk_x11_device_manager_new (GdkDisplay *display)
             {
               GdkX11DeviceManagerXI2 *device_manager_xi2;
 
-              GDK_NOTE (INPUT, g_print ("Creating XI2 device manager\n"));
+              GDK_NOTE (INPUT, g_message ("Creating XI2 device manager"));
 
               device_manager_xi2 = g_object_new (GDK_TYPE_X11_DEVICE_MANAGER_XI2,
                                                  "display", display,
@@ -70,7 +70,7 @@ _gdk_x11_device_manager_new (GdkDisplay *display)
 #endif /* XINPUT_2 */
     }
 
-  GDK_NOTE (INPUT, g_print ("Creating core device manager\n"));
+  GDK_NOTE (INPUT, g_message ("Creating core device manager"));
 
   return g_object_new (GDK_TYPE_X11_DEVICE_MANAGER_CORE,
                        "display", display,
@@ -85,7 +85,7 @@ _gdk_x11_device_manager_new (GdkDisplay *display)
  * Returns the #GdkDevice that wraps the given device ID.
  *
  * Returns: (transfer none) (allow-none) (type GdkX11DeviceCore): The #GdkDevice wrapping the device ID,
- *          or %NULL if the given ID doesn't currently represent a device.
+ *          or %NULL if the given ID doesn’t currently represent a device.
  *
  * Since: 3.2
  **/
@@ -124,13 +124,11 @@ gdk_x11_device_manager_lookup (GdkDeviceManager *device_manager,
  *
  * Returns the device ID as seen by XInput2.
  *
- * <note>
- *   If gdk_disable_multidevice() has been called, this function
- *   will respectively return 2/3 for the core pointer and keyboard,
- *   (matching the IDs for the Virtual Core Pointer and Keyboard in
- *   XInput 2), but calling this function on any slave devices (i.e.
- *   those managed via XInput 1.x), will return 0.
- * </note>
+ * > If gdk_disable_multidevice() has been called, this function
+ * > will respectively return 2/3 for the core pointer and keyboard,
+ * > (matching the IDs for the Virtual Core Pointer and Keyboard in
+ * > XInput 2), but calling this function on any slave devices (i.e.
+ * > those managed via XInput 1.x), will return 0.
  *
  * Returns: the XInput2 device ID.
  *
