@@ -1555,7 +1555,6 @@ _gdk_x11_display_open (const gchar *display_name)
   gchar *argv[1];
 
   XClassHint *class_hint;
-  gulong pid;
   gint ignore;
   gint maj, min;
 
@@ -1726,11 +1725,15 @@ _gdk_x11_display_open (const gchar *display_name)
   if (gdk_sm_client_id)
     set_sm_client_id (display, gdk_sm_client_id);
 
-  pid = getpid ();
-  XChangeProperty (display_x11->xdisplay,
-		   display_x11->leader_window,
-		   gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_PID"),
-		   XA_CARDINAL, 32, PropModeReplace, (guchar *) & pid, 1);
+  if (!gdk_running_in_sandbox ())
+    {
+      /* if sandboxed, we're likely in a pid namespace and would only confuse the wm with this */
+      pid_t pid = getpid ();
+      XChangeProperty (display_x11->xdisplay,
+                       display_x11->leader_window,
+                       gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_PID"),
+                       XA_CARDINAL, 32, PropModeReplace, (guchar *) & pid, 1);
+    }
 
   /* We don't yet know a valid time. */
   display_x11->user_time = 0;
@@ -3072,6 +3075,16 @@ gdk_x11_display_get_default_seat (GdkDisplay *display)
 
   seats = gdk_display_list_seats (display);
 
+  /* Shortcut only one seat being available.
+   * This path always triggers for core events, so we can freely use XInput below. */
+  if (g_list_length (seats) == 1)
+    {
+      GdkSeat *seat = seats->data;
+
+      g_list_free (seats);
+      return seat;
+    }
+
   gdk_x11_display_error_trap_push (display);
   result = XIGetClientPointer (GDK_DISPLAY_XDISPLAY (display),
                                None, &device_id);
@@ -3112,7 +3125,7 @@ gdk_x11_display_get_monitor (GdkDisplay *display,
 {
   GdkX11Display *x11_display = GDK_X11_DISPLAY (display);
 
-  if (0 <= monitor_num || monitor_num < x11_display->monitors->len)
+  if (0 <= monitor_num && monitor_num < x11_display->monitors->len)
     return (GdkMonitor *)x11_display->monitors->pdata[monitor_num];
 
   return NULL;
