@@ -318,7 +318,7 @@ lookup_in_global_parent_cache (GtkCssNode                  *node,
 
   g_assert (node->cache == NULL);
   node->cache = gtk_css_node_style_cache_lookup (parent->cache,
-                                                 (GtkCssNodeDeclaration *) decl,
+                                                 decl,
                                                  gtk_css_node_is_first_child (node),
                                                  gtk_css_node_is_last_child (node));
   if (node->cache == NULL)
@@ -434,6 +434,10 @@ gtk_css_node_real_update_style (GtkCssNode   *cssnode,
                                               timestamp,
                                               gtk_css_node_get_style_provider (cssnode),
                                               should_create_transitions (change) ? style : NULL);
+
+      /* Clear the cache again, the static style we looked up above
+       * may have populated it. */
+      g_clear_pointer (&cssnode->cache, gtk_css_node_style_cache_unref);
     }
   else if (static_style != style && (change & GTK_CSS_CHANGE_TIMESTAMP))
     {
