@@ -45,6 +45,7 @@ G_BEGIN_DECLS
 
 /**
  * GtkTextWindowType:
+ * @GTK_TEXT_WINDOW_PRIVATE: Invalid value, used as a marker
  * @GTK_TEXT_WINDOW_WIDGET: Window that floats over scrolling areas.
  * @GTK_TEXT_WINDOW_TEXT: Scrollable text window.
  * @GTK_TEXT_WINDOW_LEFT: Left side border window.
@@ -56,9 +57,7 @@ G_BEGIN_DECLS
  */
 typedef enum
 {
-  /*< private >*/
   GTK_TEXT_WINDOW_PRIVATE,
-  /*< public >*/
   GTK_TEXT_WINDOW_WIDGET,
   GTK_TEXT_WINDOW_TEXT,
   GTK_TEXT_WINDOW_LEFT,
@@ -164,7 +163,7 @@ struct _GtkTextViewClass
 {
   GtkContainerClass parent_class;
 
-  /*< public */
+  /*< public >*/
 
   void (* populate_popup)        (GtkTextView      *text_view,
                                   GtkWidget        *popup);
@@ -192,6 +191,7 @@ struct _GtkTextViewClass
                                   const GtkTextIter      *location,
                                   GtkTextIter            *start,
                                   GtkTextIter            *end);
+  void (* insert_emoji)          (GtkTextView      *text_view);
 
   /*< private >*/
 
@@ -200,7 +200,6 @@ struct _GtkTextViewClass
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
   void (*_gtk_reserved4) (void);
-  void (*_gtk_reserved5) (void);
 };
 
 GDK_AVAILABLE_IN_ALL
@@ -259,12 +258,12 @@ void           gtk_text_view_get_iter_location     (GtkTextView   *text_view,
                                                     const GtkTextIter *iter,
                                                     GdkRectangle  *location);
 GDK_AVAILABLE_IN_ALL
-void           gtk_text_view_get_iter_at_location  (GtkTextView   *text_view,
+gboolean       gtk_text_view_get_iter_at_location  (GtkTextView   *text_view,
                                                     GtkTextIter   *iter,
                                                     gint           x,
                                                     gint           y);
 GDK_AVAILABLE_IN_ALL
-void           gtk_text_view_get_iter_at_position  (GtkTextView   *text_view,
+gboolean       gtk_text_view_get_iter_at_position  (GtkTextView   *text_view,
                                                     GtkTextIter   *iter,
 						    gint          *trailing,
                                                     gint           x,
