@@ -100,6 +100,7 @@ static void
 gtk_printer_cups_init (GtkPrinterCups *printer)
 {
   printer->device_uri = NULL;
+  printer->original_device_uri = NULL;
   printer->printer_uri = NULL;
   printer->state = 0;
   printer->hostname = NULL;
@@ -137,6 +138,8 @@ gtk_printer_cups_init (GtkPrinterCups *printer)
   printer->sides_supported = NULL;
   printer->number_of_covers = 0;
   printer->covers = NULL;
+  printer->output_bin_default = NULL;
+  printer->output_bin_supported = NULL;
 }
 
 static void
@@ -149,6 +152,7 @@ gtk_printer_cups_finalize (GObject *object)
   printer = GTK_PRINTER_CUPS (object);
 
   g_free (printer->device_uri);
+  g_free (printer->original_device_uri);
   g_free (printer->printer_uri);
   g_free (printer->hostname);
   g_free (printer->ppd_name);
@@ -190,6 +194,9 @@ gtk_printer_cups_finalize (GObject *object)
   g_free (printer->sides_default);
   g_list_free_full (printer->sides_supported, g_free);
 
+  g_free (printer->output_bin_default);
+  g_list_free_full (printer->output_bin_supported, g_free);
+
   if (printer->get_remote_ppd_poll > 0)
     g_source_remove (printer->get_remote_ppd_poll);
   printer->get_remote_ppd_attempts = 0;
@@ -532,12 +539,17 @@ colord_client_connect_cb (GObject *source_object,
   gboolean ret;
   GError *error = NULL;
   GtkPrinterCups *printer = GTK_PRINTER_CUPS (user_data);
+  static gboolean colord_warned = FALSE;
 
   ret = cd_client_connect_finish (CD_CLIENT (source_object),
                                   res, &error);
   if (!ret)
     {
-      g_warning ("failed to contact colord: %s", error->message);
+      if (!colord_warned)
+        {
+          g_warning ("failed to contact colord: %s", error->message);
+          colord_warned = TRUE;
+        }
       g_error_free (error);
     }
 
